//+------------------------------------------------------------------+
//|                                                        DTree.mq5 |
//|                                      Copyright 2018, Dale Woods. |
//|                                      https://www.theforexguy.com |
//+------------------------------------------------------------------+
#property copyright "Copyright 2018, Dale Woods."
#property link      "https://www.theforexguy.com"
#property version   "1.00"

#include <Math/Alglib/Alglib.mqh>

enum enum_Descision_Forest_Result {
   DF_RESULT_CLASSIFICATION_INPUT_ERROR = -2,
   DF_RESULT_PARAMETERS_ERROR = -1,
   DF_RESULT_UNEXPECTED_ERROR = 0,
   DF_RESULT_PASS = 1
};


enum enum_iris_type {   
   IRIS_UNKNOWN = 0,
   IRIS_SETOSA = 1,
   IRIS_VERSICOLOR = 2,
   IRIS_VIRGINCIA = 3
};


CDForest myforest();
//+------------------------------------------------------------------+
//| Script program start function                                    |
//+------------------------------------------------------------------+
void OnStart() {
  
   CMatrixDouble inputs;
   
   //doens't work
   string path = TerminalInfoString(TERMINAL_DATA_PATH) + "\\MQL5\\Scripts\\Decision Tree Example\\iris.csv";   
   path = "iris.data";
   int colCount;         
   ReadDataFile(path, colCount, inputs);
   
   bool testDataStruct = false;
   
   if( testDataStruct ) {
      for( int i = 0; i < inputs.Size(); i++ ) { 
         CRowDouble row; 
         
         row = inputs[i];
         
         for( int col = 0; col < row.Size(); col++ ) { 
            double data;
            
            data = row[col];
            
            printf("data on col %i = %f", col, data);
         
         }      
      } 
   }
   
   
   // TRAIN DECISION TREE
   CDecisionForest dforest;
   int result;
   CDFReport report;
   
   //DFBuildRandomDecisionForest()
   //+------------------------------------------------------------------+
   //| This subroutine builds random decision forest.                   |
   //| INPUT PARAMETERS:                                                |
   //|     XY          -   training set                                 |
   //|     NPoints     -   training set size, NPoints>=1                |
   //|     NVars       -   number of independent variables, NVars>=1    |
   //|     NClasses    -   task type:                                   |
   //|                     * NClasses=1 - regression task with one      |
   //|                                    dependent variable            |
   //|                     * NClasses>1 - classification task with      |
   //|                                    NClasses classes.             |
   //|     NTrees      -   number of trees in a forest, NTrees>=1.      |
   //|                     recommended values: 50-100.                  |
   //|     R           -   percent of a training set used to build      |
   //|                     individual trees. 0<R<=1.                    |
   //|                     recommended values: 0.1 <= R <= 0.66.        |
   //| OUTPUT PARAMETERS:                                               |
   //|     Info        -   return code:                                 |
   //|                     * -2, if there is a point with class number  |
   //|                           outside of [0..NClasses-1].            |
   //|                     * -1, if incorrect parameters was passed     |
   //|                           (NPoints<1, NVars<1, NClasses<1,       |
   //|                           NTrees<1, R<=0 or R>1).                |
   //|                     *  1, if task has been solved                |
   //|     DF          -   model built                                  |
   //|     Rep         -   training report, contains error on a training|
   //|                     set and out-of-bag estimates of              |
   //|                     generalization error.                        |
   //+------------------------------------------------------------------+
   
   // printf("data size = %i", inputs.Size() );
   
   myforest.DFBuildRandomDecisionForest(inputs, inputs.Size(), colCount-1 , 4, 2, 0.9, result, dforest, report);
   
   
   /*   
   class CDFReport  
      double            m_relclserror;          // 
      double            m_avgce;                // Average cross-entropy (in bits per element) on the test set
      double            m_rmserror;             // 
      double            m_avgerror;             // Average error on the test set
      double            m_avgrelerror;          // Average relative error on the test set
      double            m_oobrelclserror;       // 
      double            m_oobavgce;             // out-of-bag average cross-entropy
      double            m_oobrmserror;    
      double            m_oobavgerror;
      double            m_oobavgrelerror;        
   */ 
   
   printf("Result from DFBuildRandomDecisionForest() = %s", EnumToString( (enum_Descision_Forest_Result) result) );
   
   printf("Average error = %f", report.m_avgerror);
   printf("Obs Average error = %f", report.m_oobavgerror);
   
   double x[4] = { 6.3,3.3,4.7,1.6 };
   double y[];
   
   myforest.DFProcess(dforest, x, y);
   
   /*
      Why DFProcess() resizes the output to match x arr, I am not sure???
   */   
   for( int i = 0; i < ArraySize(y); i++ ){
      printf("prediction output = y[%i] = %f", y[i]);  
   }
   
}
//+------------------------------------------------------------------+



/*
   Generate Random data for decision tree
*/
void PrepareData(CMatrixDouble &out) {

   int rowCount = 20;
   int colCount = 3;
   
   CMatrixDouble data(rowCount);
   
   for( int i = 0; i < rowCount; i++ ) {
      
      CRowDouble row;
      
      row.Resize(colCount);
      
      for( int col = 0; col < colCount; col++) {      
         row.Set(col, (double) MathRand());
      }
      
      data[i] = row;   
   }


   out = data;
}



/*
   Convert CSV to CMatrixDouble object
   
   @Note: CSV must only contain double values
*/
void ReadDataFile(const string csvFilePath, int &colCount, CMatrixDouble &out) {
     
   int handle = FileOpen(csvFilePath, FILE_READ|FILE_CSV|FILE_ANSI, ',');
   
   if( handle == INVALID_HANDLE ) {  printf("Couldn't open file %s, reason = %i", csvFilePath, GetLastError()); return; }
   
   // Scan first line to check how many columns there are
   colCount = 0;   
   while( !FileIsLineEnding(handle) && !FileIsEnding(handle) && !IsStopped() ) { FileReadString(handle); colCount++; }
   
   // Reset file pointer
   FileSeek(handle, 0, SEEK_SET);
   
   // Init row counter to keep track of how many data sets we have
   int rowNum = 0;
   
   while( !FileIsEnding(handle) && !IsStopped() ) {
      
      string s;
      double d;
      
      CRowDouble row;
      row.Resize(colCount);      
      
      for( int i = 0; i < colCount; i++) {
         s = FileReadString(handle);
         d = StringToDouble(s);
         row.Set(i, d);       
         //printf("Col[%i] = %s", i, s);
      }
      
      // Resize CMatrixDouble to add new row
      out.Resize( out.Size()+1, colCount );
      
      // Add the new row
      out[rowNum] = row;
      
      rowNum++;
   } 
   
   FileClose(handle);   
}


/*
   NOTES:
   
      
   
   
   
   
   

*/